@@ -7,6 +7,7 @@ our @ISA = qw/Exporter/;
 
 our %NAMESPACES = (
    client      => 'jabber:client',
+   component   => 'jabber:component:accept',
    stream      => 'http://etherx.jabber.org/streams',
    streams     => 'urn:ietf:params:xml:ns:xmpp-streams',
    stanzas     => 'urn:ietf:params:xml:ns:xmpp-stanzas',
@@ -14,12 +15,13 @@ our %NAMESPACES = (
    bind        => 'urn:ietf:params:xml:ns:xmpp-bind',
    tls         => 'urn:ietf:params:xml:ns:xmpp-tls',
    roster      => 'jabber:iq:roster',
+   register    => 'jabber:iq:register',
    version     => 'jabber:iq:version',
    session     => 'urn:ietf:params:xml:ns:xmpp-session',
    xml         => 'http://www.w3.org/XML/1998/namespace',
    disco_info  => 'http://jabber.org/protocol/disco#info',
    disco_items => 'http://jabber.org/protocol/disco#items',
-   register    => 'http://jabber.org/features/iq-register',
+   register_f  => 'http://jabber.org/features/iq-register',
    iqauth      => 'http://jabber.org/features/iq-auth',
    data_form   => 'jabber:x:data',
    muc         => 'http://jabber.org/protocol/muc',
